package digital.laboratory.platform.reagent.controller;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.plugin.table.LoopRowTableRenderPolicy;
import com.fasterxml.jackson.annotation.JsonFormat;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.feign.RemoteWord2PDFService;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.dto.ComplianceCheckDTO;
import digital.laboratory.platform.reagent.entity.ComplianceCheck;
import digital.laboratory.platform.reagent.entity.ReagentConsumables;
import digital.laboratory.platform.reagent.service.ComplianceCheckService;
import digital.laboratory.platform.reagent.service.ReagentConsumablesService;
import digital.laboratory.platform.reagent.vo.ComplianceCheckVO;
import digital.laboratory.platform.reagent.vo.ProcurementContentVO;
import digital.laboratory.platform.reagent.vo.PurchasingPlanVO;
import feign.Response;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.security.Principal;
import java.sql.Date;
import java.time.LocalDate;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;

/**
 * (符合性检查记录表)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (符合性检查记录表) 前端控制器
 * <p>
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 * 为前端提供数据, 接受前端的数据
 * 为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 * 接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/compliance_check")
@Api(value = "compliance_check", tags = "(符合性检查记录表)管理")
@SuppressWarnings("all")
public class ComplianceCheckController {

    private final ComplianceCheckService complianceCheckService;

    private final OssFile ossFile;
    @Autowired
    private RemoteWord2PDFService remoteWord2PDFService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    /**
     * 通过id查询(符合性检查记录表)
     *
     * @param complianceCheckId id
     * @return R
     */
    @ApiOperation(value = "通过id查询", notes = "通过id查询")
    @GetMapping()
    @PreAuthorize("@pms.hasPermission('reagent_compliance_check_get')")
    public R<ComplianceCheckVO> getById(String complianceCheckId, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ComplianceCheckVO complianceCheckVO = complianceCheckService.getComplianceCheckVO(complianceCheckId);

        return R.ok(complianceCheckVO);
    }

    /**
     * 分页查询
     *
     * @param page            分页对象
     * @param complianceCheck (符合性检查记录表)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page")
    @PreAuthorize("@pms.hasPermission('reagent_compliance_check_page')")
    public R<IPage<ComplianceCheckVO>> getComplianceCheckPage(Page<ComplianceCheck> page, String rid, String status, @DateTimeFormat(pattern = "yyyy-MM-dd") @JsonFormat(pattern = "yyyy-MM-dd") LocalDate startTime, @DateTimeFormat(pattern = "yyyy-MM-dd") @JsonFormat(pattern = "yyyy-MM-dd") LocalDate endTime, String reagentConsumableName, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        if (StrUtil.isNotBlank(rid)) {

            IPage<ComplianceCheckVO> complianceCheckList = complianceCheckService.getComplianceCheckVOPage(page, Wrappers.<ComplianceCheck>query()
                    .eq("reference_material_id", rid)
                    .eq("status", 6)
                    .orderByDesc("create_time"));
            return R.ok(complianceCheckList);
        }

        IPage<ComplianceCheckVO> complianceCheckSList = complianceCheckService.getComplianceCheckVOPage(page, Wrappers.<ComplianceCheck>query()
                .like(StrUtil.isNotBlank(reagentConsumableName), "reagent_consumable_name", reagentConsumableName)
                .ge(startTime != null, "create_time", startTime)
                .le(endTime != null, "create_time", endTime)
                .eq(StrUtil.isNotBlank(status), "status", status)
                .orderByDesc("create_time")

        );


        return R.ok(complianceCheckSList);
//		return R.ok(complianceCheckService.page(page, Wrappers.query(complianceCheck)));
    }


//    /**
//     * 新增符合性检查记录表(试剂耗材)
//     *
//     * @param complianceCheckDTO (符合性检查记录表)
//     * @return R
//     */
//    @ApiOperation(value = "新增符合性检查记录表(试剂耗材)", notes = "新增符合性检查记录表(试剂耗材)")
//    @SysLog("新增(符合性检查记录表)")
//    @PostMapping
////    @PreAuthorize("@pms.hasPermission('reagent_compliance_check_add')")
//    public R<ComplianceCheck> postAddObject(@RequestBody ComplianceCheckDTO complianceCheckDTO, HttpServletRequest theHttpServletRequest) {
//        Principal principal = theHttpServletRequest.getUserPrincipal();
//        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
//
//        ComplianceCheck complianceCheck = complianceCheckService.addCheck(complianceCheckDTO, dlpUser);
//
//        if (complianceCheck != null) {
//            return R.ok(complianceCheck, "保存成功");
//        } else {
//            return R.failed(complianceCheck, "保存失败");
//        }
//    }

    /**
     * 手动新增符合性检查记录表(试剂耗材)
     *
     * @param complianceCheckDTO (手动新增符合性检查记录表)
     * @return R
     */
    @ApiOperation(value = "手动新增符合性检查记录表(试剂耗材)", notes = "手动新增符合性检查记录表(试剂耗材)")
    @SysLog("新增(符合性检查记录表)")
    @PostMapping("/addList")
    @PreAuthorize("@pms.hasPermission('reagent_compliance_check_add')")
    public R<List<ComplianceCheck>> addCheckList(@RequestBody List<ComplianceCheckDTO> complianceCheckDTOList, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        List<ComplianceCheck> complianceCheckList = complianceCheckService.addCheckList(complianceCheckDTOList, dlpUser);

        if (complianceCheckList != null) {
            return R.ok(complianceCheckList, "保存成功");
        } else {
            return R.failed(complianceCheckList, "保存失败");
        }
    }


    /**
     * 通过id删除(符合性检查记录表)
     *
     * @param complianceCheckId id
     * @return R
     */
    @ApiOperation(value = "通过id删除(符合性检查记录表)", notes = "通过id删除(符合性检查记录表)")
    @SysLog("通过id删除(符合性检查记录表)")
    @DeleteMapping()
    @PreAuthorize("@pms.hasPermission('reagent_compliance_check_del')")
    public R<String> deleteById(String complianceCheckId, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ;

        if (complianceCheckService.delCheckById(complianceCheckId)) {
            return R.ok("删除成功");
        } else {
            return R.failed("删除失败");
        }

    }

    /**
     * 编辑符合性检查记录表（录入结果）
     *
     * @param complianceCheckId id
     * @return R
     */
    @ApiOperation(value = "编辑符合性检查记录表", notes = "编辑符合性检查记录表")
    @SysLog("编辑符合性检查记录表")
    @PutMapping("/edit")
    @PreAuthorize("@pms.hasPermission('reagent_compliance_check_edit')")
    public R<String> editCheckById(@RequestBody ComplianceCheckDTO complianceCheckDTO, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ComplianceCheck complianceCheck = complianceCheckService.editCheckById(complianceCheckDTO);

        if (complianceCheck != null) {
            return R.ok("保存成功");
        } else {
            return R.failed("保存失败");
        }

    }

    /**
     * 录入符合性检查方案
     *
     * @param complianceCheckId id
     * @return R
     */
    @ApiOperation(value = "录入符合性检查方案", notes = "录入符合性检查方案")
    @SysLog("编辑符合性检查记录表")
    @PutMapping("/addScheme")
    @PreAuthorize("@pms.hasPermission('reagent_compliance_check_input')")
    public R<String> addScheme(@RequestBody ComplianceCheckDTO complianceCheckDTO, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ComplianceCheck complianceCheck = complianceCheckService.addScheme(complianceCheckDTO, dlpUser);

        if (complianceCheck != null) {
            return R.ok("保存成功");
        } else {
            return R.failed("保存失败");
        }

    }

    /**
     * 提交符合性检查记录表
     *
     * @param complianceCheckId id
     * @return R
     */
    @ApiOperation(value = "提交符合性检查记录表", notes = "提交符合性检查记录表")
    @SysLog("提交符合性检查记录表")
    @PutMapping("/commit")
    @PreAuthorize("@pms.hasPermission('reagent_compliance_check_commit')")
    public R<String> commitCheck(@RequestBody ComplianceCheckDTO complianceCheckDTO, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ComplianceCheck complianceCheck = complianceCheckService.commitCheck(complianceCheckDTO, dlpUser);

        if (complianceCheck != null) {
            return R.ok("提交成功");
        } else {
            return R.failed("提交失败");
        }

    }

    /**
     * 一级审核符合性检查记录表
     *
     * @param complianceCheckId id
     * @return R
     */
    @ApiOperation(value = "一级审核符合性检查记录表", notes = "一级审核符合性检查记录表")
    @SysLog("一级审核符合性检查记录表")
    @PutMapping("/primaryAudit")
    @PreAuthorize("@pms.hasPermission('reagent_compliance_check_primaryAudit')")
    public R<String> primaryAuditCheck(@RequestBody AuditAndApproveDTO auditAndApproveDTO, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        ComplianceCheck complianceCheck = complianceCheckService.primaryAuditCheck(auditAndApproveDTO, dlpUser);

        if (complianceCheck != null) {
            return R.ok("审核成功");
        } else {
            return R.failed("审核失败");
        }

    }

    /**
     * 二级审核符合性检查记录表
     *
     * @param complianceCheckId id
     * @return R
     */
    @ApiOperation(value = "二级审核符合性检查记录表", notes = "二级审核符合性检查记录表")
    @SysLog("二级审核符合性检查记录表")
    @PutMapping("/secondaryAudit")
    @PreAuthorize("@pms.hasPermission('reagent_compliance_check_secondaryAudit')")
    public R<String> secondaryAuditCheck(@RequestBody AuditAndApproveDTO auditAndApproveDTO, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        ComplianceCheck complianceCheck = complianceCheckService.secondaryAuditCheck(auditAndApproveDTO, dlpUser);
        if (complianceCheck != null) {
            return R.ok("审核成功");
        } else {
            return R.failed("审核失败");
        }

    }

    /**-------采购计划打印*/
    /**
     * 采购计划打印
     *
     * @param purchasingPlanId
     * @return
     */
    @ApiOperation(value = "试剂耗材符合性检查打印", notes = "试剂耗材符合性检查打印")
    @SysLog("试剂耗材符合性检查打印")
    @PostMapping("/print")
    @PreAuthorize("@pms.hasPermission('reagent_compliance_check_print')")
    public void bizGetPDFInventory(String complianceCheckId, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) {
        System.out.println("bizApplyWord.................");
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ComplianceCheckVO complianceCheckVO = complianceCheckService.getComplianceCheckVO(complianceCheckId);

        String id = complianceCheckVO.getId();

        String applyFileName = null;

        if (reagentConsumablesService.getById(complianceCheckVO.getReagentConsumableId()).getCategory().equals("标准物质") |
                reagentConsumablesService.getById(complianceCheckVO.getReagentConsumableId()).getCategory().equals("标准储备溶液")) {
            applyFileName = "标准物质符合性检查" + id;
        } else {
            applyFileName = "试剂耗材符合性检查-" + id;
        }

        String pdfFilePath = "document" + "/" + "complianceCheck" + "/" + id + "/" + applyFileName + ".pdf";
        try {
            //直接调用pdf方法
            complianceCheckTablePDF(complianceCheckVO, id, theHttpServletRequest, httpServletResponse);
            ossFile.fileGet(pdfFilePath, httpServletResponse.getOutputStream());
            httpServletResponse.setContentType(applyFileName);
        } catch (Exception e) {
            System.out.println(String.format("minioFile objectExist() Exception. %s", e.getLocalizedMessage()));
            e.printStackTrace();
        }
    }

    public void complianceCheckTablePDF(ComplianceCheckVO complianceCheckVO, String id, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {

        System.out.println("ComplianceCheckTablePDF.................");

        String applyFileName = null;
        //-----------------------------
        // 生成 word 版本的 采购目录


        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ReagentConsumables byId = reagentConsumablesService.getById(complianceCheckVO.getReagentConsumableId());

        if (byId.getCategory().equals("标准物质") |
                byId.getCategory().equals("标准储备溶液")) {
            ossFile.fileGet("template" + "/" + "标准物质符合性检查记录表模板.docx", bos);
            applyFileName = "标准物质符合性检查" + id;
        } else {
            ossFile.fileGet("template" + "/" + "试剂耗材符合性检查记录表模板.docx", bos);
            applyFileName = "试剂耗材符合性检查-" + id;
        }


        byte[] templateArray = bos.toByteArray();

        ByteArrayInputStream bis = new ByteArrayInputStream(templateArray);
        bos.close();

        LoopRowTableRenderPolicy policy = new LoopRowTableRenderPolicy();

        Configure config = Configure.builder().
                bind("voList", policy).build();

        Calendar instance = Calendar.getInstance();
        int year = instance.get(Calendar.YEAR);
        /**
         * 循环表格
         */
        XWPFTemplate template = XWPFTemplate.compile(bis, config).render(
                new HashMap<String, Object>() {{
                    put("dateOfInspection", LocalDateTimeUtil.format(complianceCheckVO.getDateOfInspection(), "yyyy年MM月dd日"));
                    put("year", year);
                    put("reagentConsumableName", complianceCheckVO.getReagentConsumableName());
                    put("brand", complianceCheckVO.getBrand());
                    put("specificationAndModel", complianceCheckVO.getSpecificationAndModel());
                    put("inspectionScheme", complianceCheckVO.getInspectionScheme());
                    put("examinationConclusion", complianceCheckVO.getExaminationConclusion());
                    put("nonconformingItem", complianceCheckVO.getNonconformingItem());
                    put("primaryAuditorName", complianceCheckVO.getPrimaryAuditorName());
                    put("secondaryAuditorName", complianceCheckVO.getSecondaryAuditorName());
                    put("auditTimeOfPrimary", LocalDateTimeUtil.format(complianceCheckVO.getAuditTimeOfPrimary(), "yyyy年MM月dd日"));
                    put("auditTimeOfSecondary", LocalDateTimeUtil.format(complianceCheckVO.getAuditTimeOfSecondary(), "yyyy年MM月dd日"));

                    if (byId.getCategory().equals("标准物质") |
                            byId.getCategory().equals("标准储备溶液")) {
                        put("executorName", complianceCheckVO.getExecutorName());
                        put("reagentConsumableNumber", complianceCheckVO.getReagentConsumableNumber());
                        put("commitTime", LocalDateTimeUtil.format(complianceCheckVO.getCommitTime(), "yyyy年MM月dd日"));
                    }
                }}
        );
        bis.close();

        ByteArrayOutputStream fosWord = new ByteArrayOutputStream();
        template.write(fosWord);
        template.close();

        //------------
        ByteArrayInputStream fisWord = new ByteArrayInputStream(fosWord.toByteArray());
        fosWord.close();

        //MockMultipartFile mockMultipartFile = new MockMultipartFile("file", entrustmentLetterFileName + ".docx", "application/vnd.openxmlformats-officedocument.wordprocessingml.document", fisWord);
        MockMultipartFile mockMultipartFile = new MockMultipartFile("file", applyFileName + ".docx", "image/jpg", fisWord);
        Response response = remoteWord2PDFService.word2pdf(mockMultipartFile);
        fisWord.close();


        ByteArrayOutputStream outPDF = new ByteArrayOutputStream();
        IoUtil.copy(response.body().asInputStream(), outPDF, IoUtil.DEFAULT_MIDDLE_BUFFER_SIZE);
        ByteArrayInputStream isPDF = new ByteArrayInputStream(outPDF.toByteArray());
        outPDF.close();


        ossFile.fileSave("document" + "/" + "complianceCheck" + "/" + id + "/" + applyFileName + ".pdf", isPDF);
        isPDF.close();


        System.out.println(String.format("转换为 PDF 结束"));

    }
}
