package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.Blacklist;
import digital.laboratory.platform.reagent.entity.ProvideServicesOrSupplies;
import digital.laboratory.platform.reagent.service.BlacklistService;
import digital.laboratory.platform.reagent.service.ProvideServicesOrSuppliesService;
import digital.laboratory.platform.reagent.vo.BlackListVO;
import digital.laboratory.platform.reagent.vo.ProvideServicesOrSuppliesVO;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;
import java.util.List;

/**
 * 提供服务或供应品
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe 提供服务或供应品 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/provide_services_or_supplies" )
@Api(value = "provide_services_or_supplies", tags = "提供服务或供应品管理")
public class ProvideServicesOrSuppliesController {

    private final  ProvideServicesOrSuppliesService provideServicesOrSuppliesService;

	private final BlacklistService blacklistService;

	/**
	 * 通过id查询提供服务或供应品
	 * @param evaluationFormId id
	 * @return R
	 */
	@ApiOperation(value = "通过供应商id查询该供应商提供的所有物品", notes = "通过供应商id查询该供应商提供的所有物品")
	@GetMapping()
//	@PreAuthorize("@pms.hasPermission('reagent_provide_services_or_supplies_get')" )
	public R<List<ProvideServicesOrSuppliesVO>> getById(String evaluationFormId, HttpServletRequest theHttpServletRequest) {

		Principal principal = theHttpServletRequest.getUserPrincipal();

		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		List<ProvideServicesOrSuppliesVO> voList = provideServicesOrSuppliesService.getVOList(evaluationFormId);
		return R.ok(voList);
		//return R.ok(provideServicesOrSuppliesService.getById(provideServicesOrSuppliesid));
	}

	/**
	 * 通过id查询提供服务或供应品
	 * @param supplierInformationId id
	 * @return R
	 */
	@ApiOperation(value = "通过供应商id不符合内容", notes = "通过供应商id不符合内容")
	@GetMapping("/blackList")
//	@PreAuthorize("@pms.hasPermission('reagent_provide_services_or_supplies_get')" )
	public R<List<BlackListVO>> getBlackList(String supplierInformationId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		List<BlackListVO> blacklist = blacklistService.getBlacklist(supplierInformationId);

		return R.ok(blacklist);
		//return R.ok(provideServicesOrSuppliesService.getById(provideServicesOrSuppliesid));
	}



}
