package digital.laboratory.platform.reagent.task;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import digital.laboratory.platform.reagent.entity.BatchDetails;
import digital.laboratory.platform.reagent.entity.ReagentConsumableInventory;
import digital.laboratory.platform.reagent.entity.ReferenceMaterial;
import digital.laboratory.platform.reagent.entity.StandardReserveSolution;
import digital.laboratory.platform.reagent.service.*;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;

@Configuration
@EnableScheduling
@RequiredArgsConstructor
public class MaturityCalculation {

    private final BatchDetailsService batchDetailsService;

    private final PeriodVerificationPlanService periodVerificationPlanService;

    private final ReagentConsumableInventoryService reagentConsumableInventoryService;

    private final StandardReserveSolutionService standardReserveSolutionService;

    private final ReferenceMaterialService referenceMaterialService;


    @Scheduled(cron = "1 0 0 * * ?  ")
    public void calculate() {
        //查找出有库存量的物品所有批次信息(状态为1)
        List<BatchDetails> list1 = batchDetailsService.list(Wrappers.<BatchDetails>query().eq("service_status", 1));
        String warningInformation = null;
        for (BatchDetails batchDetails : list1) {
            //提前一周进行到期提醒
            if (batchDetails.getExpirationDate().isBefore(LocalDate.now().plusDays(7))) {
                warningInformation = "即将过期";
                batchDetails.setWarningInformation(warningInformation);
                batchDetailsService.updateById(batchDetails);
            }

            if (batchDetails.getExpirationDate().isBefore(LocalDate.now())) {
                warningInformation = "已过期";
                batchDetails.setWarningInformation(warningInformation);
                batchDetailsService.updateById(batchDetails);
            }
        }
        List<StandardReserveSolution> list = standardReserveSolutionService.list(Wrappers.<StandardReserveSolution>query()
                .eq("status", 1));

        for (StandardReserveSolution standardReserveSolution : list) {

            ReferenceMaterial referenceMaterialServiceById = referenceMaterialService.getById(standardReserveSolution.getReferenceId());

            standardReserveSolutionService.updateById(standardReserveSolution);

        }

        periodVerificationPlanService.getPlan();
        System.out.println("期间核查计划自动更新");
    }


    @Scheduled(cron = "0 0/1 * * * ?  ")
    public void inventoryReminder() {

        List<StandardReserveSolution> list2 = standardReserveSolutionService.list(Wrappers.<StandardReserveSolution>query()
                .eq("status", 1));

        for (StandardReserveSolution standardReserveSolution : list2) {

            ReferenceMaterial referenceMaterialServiceById = referenceMaterialService.getById(standardReserveSolution.getReferenceId());
            if (referenceMaterialServiceById.getStatus() == -4) {

                standardReserveSolution.setWarningInformation("已全部使用完毕或已报废");
            }

        }

        List<ReagentConsumableInventory> list = reagentConsumableInventoryService.list(Wrappers.<ReagentConsumableInventory>query().ne("category", "标准储备溶液"));

        for (ReagentConsumableInventory reagentConsumableInventory : list) {

            if (reagentConsumableInventory.getTotalQuantity() == 0) {

                reagentConsumableInventory.setWarningInformation("暂无库存");
            }

            if (reagentConsumableInventory.getTotalQuantity() <= reagentConsumableInventory.getWarningValue()) {

                reagentConsumableInventory.setWarningInformation("库存不足");

            } else {
                reagentConsumableInventory.setWarningInformation("库存充足");
            }
            reagentConsumableInventoryService.updateById(reagentConsumableInventory);
        }

        List<ReagentConsumableInventory> list1 = reagentConsumableInventoryService.list(Wrappers.<ReagentConsumableInventory>query().eq("category", "标准储备溶液"));

        for (ReagentConsumableInventory reagentConsumableInventory : list1) {

            reagentConsumableInventory.setWarningInformation("无");

            reagentConsumableInventoryService.updateById(reagentConsumableInventory);
        }
    }
}
